searchNodes=[{"doc":"Implementation of the &lt;~ extructure operator.","ref":"Extructure.html","title":"Extructure","type":"module"},{"doc":"Destructures the right hand side expression into and according to the left side expression. Supports destructure-like implicit keys (with the same name as the variable) as well as optional variables, flexible keyword and key-pair tuple size and order of elements, implicit transformation between a map, list and a key pair tuple. Also supports toggling between the loose mode and the standard Elixir pattern matching (&quot;rigid&quot;) mode where none of the flexibilities except for the optional variables are allowed. Fully enforces pattern matching between the left and right side once taken into account the optional variables and structural transformation. When using optional variables their key/value pairs are merged with the right hand side before applying the assignment. Features Optional variables - Prefix the variable name with an underscore and/or declare it as a function with zero arguments. If used, the underscore is trimmed from the variable's name e.g. an _a is translated to a . Optional variables - Declare the variable as a function taking the default value as its single argument. In the loose (default) mode use the maps, keywords and key-pair tuples interchangeably as you see fit. Toggle the loose mode into the rigid mode by using the unary ^ operator left to the map, list or tuple that requires Elixir-like pattern matching. If requiring a loose matching again at a nested level in the structure, use the ^ operator again to toggle back to loose and so on. To match typical tuples, switch to the rigid mode with the ^ operator, if only for the tuple in question. For nesting, use the keys as you would in plain Elixir matching: %{ a : %{ b } } &lt;~ [ a : %{ b : 2 } ] # a variable is not set # b variable is set to 2 or %{ a : a = %{ b } } &lt;~ [ a : %{ b : 2 } ] # both a and b variables are set Note Variables with specified keys must be (in a keyword list) trailing those without the explicit keys: %{ a , b , c : c , d : _d } &lt;~ %{ a : 1 , b : 2 , c : 3 } or [ a , b , c : c , d : _d ] &lt;~ %{ a : 1 , b : 2 , c : 3 } Any errors in the left-side expression are detected in compile time. Usage Instead of: %{ first_name : fist_name , last_name : last_name , } = socket . assigns age = socket . assigns [ :age ] simply use: %{ first_name , last_name , _age } &lt;~ socket . assigns or { first_name , last_name , _age ( 25 ) } &lt;~ socket . assigns or [ first_name , last_name , age ( 25 ) ] &lt;~ socket . assigns See the README.md and extructure_test.ex files for more examples.","ref":"Extructure.html#%3C~/2","title":"Extructure.<~/2","type":"macro"},{"doc":"Extructure is a flexible destructure library for Elixir. By default the library is using loose (flexible) matching, allowing for implicit structural conversions (maps, lists and tuples, from one to another). Tuple and list key pair element order are also taken loosely by default. Toggling from loose to Elixir-default (&quot;rigid&quot;) mode is done via the ^ operator. Optional variables are also supported with or without a default value.","ref":"readme.html","title":"Extructure","type":"extras"},{"doc":"def deps do [ { :extructure , &quot;~&gt; 0.1.0&quot; } ] end Import the Extructure module in every module where you use it: defmodule Foo do # uses, aliases, and imports import Extructure # .. end","ref":"readme.html#installation","title":"Extructure - Installation","type":"extras"},{"doc":"The docs can be found at HexDocs .","ref":"readme.html#docs","title":"Extructure - Docs","type":"extras"},{"doc":"Fetching two mandatory variables and one optional from the LiveView assigns Assuming a map of socket assigns, a standard pattern matching followed by retrieving an optional variable as shown below: %{ first_name : fist_name , last_name : last_name , } = socket . assigns age = socket . assigns [ :age ] is a one-liner in Extructure: %{ first_name , last_name , _age } &lt;~ socket . assigns Implicit transformation between maps, lists and tuples with key pairs Given the Extructure's loose treatment of structures in terms of their interchangeability, the former can be expressed in a more readable manner: { first_name , last_name , _age } &lt;~ socket . assigns or [ first_name , last_name , _age ] &lt;~ socket . assigns Default values An optional variable can be written as a function taking zero or one arguments, with a single argument being the default value, and/or as the variable name prefixed with a single underscore character _ . [ first_name , last_name , age ( 25 ) ] &lt;~ socket . assigns or [ first_name , last_name , _age ( 25 ) ] &lt;~ socket . assigns Flexible keyword list and tuple of key pairs size and element order [ b , a ] &lt;~ [ a : 1 , b : 2 , c : 3 ] # =&gt; [ b: 2, a: 1] { b , a } &lt;~ { { :a , 1 } , { :b , 2 } , { :c , 3 } } # =&gt; { { :b, 2}, { :a, 1}} Flexible keyword list head | tail extraction [ b | rest ] &lt;~ [ a : 1 , b : 2 , c : 3 ] # =&gt; [ b: 2, a: 1, c: 3] [ b | [ a , c ] ] &lt;~ [ a : 1 , b : 2 , c : 3 , d : 4 ] # =&gt; [ b: 2, a: 1, c: 3] [ a | [ b , c ( 25 ) ] ] &lt;~ %{ a : 1 , b : 2 } # =&gt; [a: 1, b: 2, c: 25] [ b | %{ c : %{ d } } ] &lt;~ [ a : 1 , b : 2 , c : %{ d : 5 } ] # =&gt; [ { :b, 2} | %{ a: 1, c: %{ d: 5}} Enforcing &quot;rigid&quot; (Elixir default) matching of the structures The rigid approach is useful to ensure the Elixir-like matching of the right side, and necessary if deconstructing standard Elixir tuples. ^ { a , b , c } &lt;~ { 1 , 2 , 3 } # ok ^ { b , a } &lt;~ { { :a , 1 } , { :b , 2 } , { :c , 3 } } # error ^ [ b , a ] &lt;~ [ a : 1 , b : 2 , c : 3 ] # error ^ %{ a } &lt;~ %{ a : 1 , b : 2 } # ok Nesting Any level of nesting is supported, and with the ^ operator toggling from loose to rigid and vice versa, any matching combination can be achieved. Ex from the extructure_test.ex : %{ a , b : b = ^ { c , d , ^ %{ e } } } &lt;~ [ a : 1 , b : { 3 , 4 , [ e : 5 ] } ] assert a == 1 assert b == { 3 , 4 , %{ e : 5 } } assert c == 3 assert d == 4 assert e == 5","ref":"readme.html#sample-usage","title":"Extructure - Sample usage","type":"extras"},{"doc":"Optional variables and default values The original idea was to use the standard \\\\ operator to denote optional and default variables in lists, tuples, and maps, but, as shown below, the Elixir parser does not support this expression in maps. [ a , b \\\\ nil ] # ok { a , b \\\\ nil } # ok %{ a , b \\\\ nil } # syntax error So, decision was made that, until there's a progress with the Elixir parser, the underscore prefixed variable names will be used for optional variables defaulting to nil, and function (macro) call syntax will be used for the optional variables defaulting to nil or any other value, e.g.: %{ _a } # optional variable, defaults to nil %{ a ( ) } # ditto %{ a ( 25 ) } # optional variable, defaults to 25 %{ _a ( 25 ) } # ditto The above syntax is used uniformly with all three types (maps, lists, tuples). The limitation that comes with this approach is that there can't be macro calls placed within the left-side expression. Should the Elixir core team decide to remove the parser restriction, a support for the standard Elixir optional variables (arguments) would be added and the present notation would be slowly phased out (leaving it to the compatibility mode).","ref":"readme.html#limitations","title":"Extructure - Limitations","type":"extras"},{"doc":"The source code formatting in this library diverges from the standard formatting practice based on using mix format in so much that there's a leading space character inserted before each initial argument / element with an intention to improve the code readability (subject to the author's personal perception). Another detail diverging from the standard Elixir formatting is that, where present, multi-line function signatures and with statements will not have the do at the end of the last of the lines but, instead, indented in the new line, e.g.: with { _ , foo } &lt;- get_foo ( a , b , c ) , { _ , bar } &lt;- foo_to_bar ( foo ) do # logic else _ -&gt; x end The preferred width is 120 characters for the code and 80 characters for the docs.","ref":"readme.html#formatting","title":"Extructure - Formatting","type":"extras"}]